#include <stdio.h>
#include <pdflib.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "debug.h"
#include "document.h"

#include "mf_cob_expand.h"

char	*myfgets( char *b, int size, FILE *f );
int	usage( const char *name );

char	*myfgets( char *b, int size, FILE *f )
{
	int	cnt;

	/* Read in up to size chars from f */
	bzero( b, size );
	for( cnt = 0; cnt < size; cnt++ )
	{
		b[ cnt ] = (char)fgetc( f );
		if( b[ cnt ] == 0 )
		{
			/* Get rid of nulls */
			b[ cnt ] = ' ';
		}

		dmsg( 6, "Read character '%c'", b[cnt] );

		if( b[ cnt ] == '\n' )
		{
			dmsg( 4, "Returning, eol" );
			return( b );
		}

		if( feof( f ) )
		{
			dmsg( 4, "Returning, eof" );
			return( b );
		}
	}
	return( b );	
}

int	usage( const char *name )
{
	printf(	"Indexed PDF Creator V." VERSION "\n"
		"Author: Steve Slaven <bpk@hoopajoo.net>  -  http://hoopajoo.net\n"
		"Usage: %s [-hDiobdTLlcfFm] [inputfile]\n"
		" -h\n"
		"	This help message\n"
		" -D LEVEL\n"
		"	Set debug level to LEVEL\n"
		" -i LINE:START:STOP[:SUPPRESS:REPARENT]\n"
		"	Index line #LINE starting at char START and stopping at STOP.\n"
		"	See the man page for details about indexing.\n"
		" -I PAGE:LINE:START:STOP[:SUPPRESS:REPARENT]\n"
		"	Create an absolute index at page PAGE on line LINE from START\n"
		"	to STOP chars.  See the man page for details about indexing.\n"
		" -o FILENAME\n"
		"	Output filename, if not set STDOUT is used\n"
		" -b FILENAME\n"
		"	Background image filename, must be a TIFF image\n"
		" -d DPI\n"
		"	Background image DPI, default is 72\n"
		" -T NUMBER\n"
		"	Set top margin\n"
		" -L NUMBER\n"
		"	Set left margin\n"
		" -l NUMBER\n"
		"	Max number of lines per page\n"
		" -c NUMBER\n"
		"	Max number of chars per line\n"
		" -m[letter-p|letter-l]\n"
		"	Page mode, letter-p is portrait, letter-l is landscape, 8.5x11\n"
		"	US letter size paper\n"
		" -f NUMBER\n"
		"	Font size\n"
		" -F[Courier]\n"
		"	Font face name\n"
		, name, DEFAULT_REPARENT );
#ifdef DEBUG_A_LOO
	printf( "DEBUG enabled\n" );
#endif

#ifdef MF_COB_EXPAND
	printf( "MF Cobol expansion enabled\n" );
#endif

	exit( 1 );
}

int 	main( int argc, char *argv[] )
{
	document	d;
	indexd		i;
	int		tmp, tmp2;
	FILE		*fin = NULL;
	char		*tmpc, *home;
 
	/* Default to stdin/out.  We let good ol' linux clean up these
	   extra chars. */ 
	char	
		*input_filename,
		line[ 255 ],
		dash_fname[ 3 ] = "-";

	/* Initialize vars */
	/* debug_level( 5 ); */

	input_filename = dash_fname;
	document_init( &d );

	/* Global config */
	parse_config( &d, "/etc/ipdf.conf" );

	home = getenv( "HOME" );
	if( home ) 
	{
		tmpc = (char *)malloc( strlen( home ) + 10 );
		strcpy( tmpc, home );
		strcat( tmpc, "/.ipdfrc" );
		parse_config( &d, tmpc );
		free( tmpc );
	}
	parse_config( &d, "./ipdf.conf" );

	/* Parse opts */
	while( ( tmp = getopt( argc, argv, "hD:i:I:o:b:d:T:L:l:c:f:F:m:" ) ) != EOF )
	{
		switch( tmp )
		{
			case 'h':
				usage( argv[ 0 ] );
				break;

			case 'd':
				/* Image dpi */
				sscanf( optarg, "%f", &d.image_dpi );
				break;

			case 'b':
				/* Image filename */
				d.image_filename = optarg;
				break;

			case 'D':
				sscanf( optarg, "%i", &tmp2 );
				debug_level( tmp2 );
				break;

			case 'I':
				i.page_no = 0;
				i.index_type = IDX_ABSOLUTE;
				i.line_no = 1;
				i.start_char = 1;
				i.stop_char = d.chars_per_line;
				i.suppress_spaces = 1;
				i.reparent = 1;
				sscanf( optarg, "%i:%i:%i:%i:%i:%i",
					&i.page_no,
					&i.line_no,
					&i.start_char,
					&i.stop_char,
					&i.suppress_spaces,
					&i.reparent );
				document_add_index( &d, &i );
				break;

			case 'i':
				i.page_no = 0;
				i.index_type = IDX_RELATIVE;
				i.line_no = 1;
				i.start_char = 1;
				i.stop_char = d.chars_per_line;
				i.suppress_spaces = 1;
				i.reparent = DEFAULT_REPARENT;
				sscanf( optarg, "%i:%i:%i:%i:%i",
					&i.line_no,
					&i.start_char,
					&i.stop_char,
					&i.suppress_spaces,
					&i.reparent );
				document_add_index( &d, &i );
				break;

			case 'o':
				d.output_filename = optarg;
				break;

			case 'm':
				/* page mode */
				if( strcmp( optarg, "letter-p" ) == 0 )
				{
					d.width = letter_width;
					d.height = letter_height;
				}else if( strcmp( optarg, "letter-l" ) == 0 )
				{
					d.width = letter_height;
					d.height = letter_width;
				}else
				{
					dmsg( 0, "Unknown page mode: '%s'", optarg );
				}
				break;

			case 'l':
				/* Lines per page */
				sscanf( optarg, "%i", &d.lines_per_page );
				break;

			case 'c':
				/* Chars per line */
				sscanf( optarg, "%i", &d.chars_per_line );
				break;

			case 'f':
				/* Font size */
				sscanf( optarg, "%f", &d.font_size );
				break;

			case 'F':
				/* Font face */
				d.font_face = optarg;
				break;

			case 'T':
				/* Set top margin */
				sscanf( optarg, "%f", &d.top_margin );
				break;

			case 'L':
				/* Left margi */
				sscanf( optarg, "%f", &d.left_margin );
				break;
		}
	}

	if( argc != optind )
	{
		input_filename = argv[ optind ];
	}
 
	/* Open in */
	if( strcmp( input_filename, "-" ) == 0 )
	{
		dmsg( 4, "Reading from STDIN" );
		fin = stdin;
	}else
	{
		fin = fopen( input_filename, "rb" );
	}

	if( ! fin )
	{
		perror( input_filename );
		return( 0 );
	}
 
	/* Init lib */
	PDF_boot();
  
	/* Create a new doc */
	document_start( &d );

	/* Reset some infos */
	PDF_set_info( d.pdf, "Subject", "Document" );
	PDF_set_info( d.pdf, "Title", input_filename );
	PDF_set_info( d.pdf, "Creator", "ipdf V." VERSION );
	PDF_set_info( d.pdf, "Author", "ipdf by Steve Slaven <bpk@hoopajoo.net>" );

	/* Read input, make output, drink beer. */
	while( ! feof( fin ) )
	{
		bzero( line, 255 );
		myfgets( line, 255, fin );

#ifdef MF_COB_EXPAND
		/* This handles expansion of an RLE encoding in 'compressed' print
			files from MF COBOL */
		mf_cob_expand( line );
#endif
		document_addline( &d, line );	
	}

	document_end( &d );

	PDF_shutdown();
  
	return( 0 );
}
